<?php
/**
 * AlxdDataTree class file.
 * @author Alexander Shchetinin <a.shchetinin@gmail.com>
 * @copyright Copyright &copy; Alexander Shchetinin 2012
 * @license http://www.opensource.org/licenses/bsd-license.php New BSD License
 * @package none
 */

/**
 * AlxdDataTree widget.
 */

class AlxdDataTree extends CWidget
{
	private $_assets;

	public $data;

	public $idField;
	public $titleField;
	public $levelField;
	public $countField;

	public $showCount = true;

	public $actions = array(
		array('label'=>'View', 'action'=>'view'),
		array('label'=>'Edit', 'action'=>'update'),
		array('label'=>'Delete', 'action'=>'delete'),
		array('label'=>'Add child', 'action'=>'addchild'),
		array('label'=>'Remove childs', 'action'=>'removechilds', 'for'=>'parent')
		);

	public $cssFile = 'AlxdDataTree.css';

	public function init()
	{
		if ($this->_assets == null)
		{
			$path = dirname(__FILE__).DIRECTORY_SEPARATOR.'assets';
			$this->_assets = Yii::app()->assetManager->publish($path);
		}

		$cssFile = $this->_assets.DIRECTORY_SEPARATOR.$this->cssFile;
		Yii::app()->clientScript->registerCssFile($cssFile);

		$jsFile = $this->_assets.DIRECTORY_SEPARATOR.'AlxdDataTree.js';
		Yii::app()->clientScript->registerScriptFile($jsFile);

		if (is_null($this->data))
			throw new CDbException(Yii::t('yiiext','The option data must be not null.'));

		if (is_null($this->idField))
			throw new CDbException(Yii::t('yiiext','The option idField must be not null.'));

		if (is_null($this->titleField))
			throw new CDbException(Yii::t('yiiext','The option titleField must be not null.'));

		if (is_null($this->levelField))
			throw new CDbException(Yii::t('yiiext','The option levelField must be not null.'));

		if (is_null($this->countField))
			throw new CDbException(Yii::t('yiiext','The option countField must be not null.'));
	}

	public function run()
	{
		echo '<div>';
		$this->renderTree($this->data);
		echo '</div>';
	}

	protected function renderTree($rows)
	{
		$prevLevel = -1;

		echo '<ul class="alxd-tree">';
		foreach ($rows as $row)
		{
			$currLevel = $row[$this->levelField];
			$cnt = $row[$this->countField];

			if ($prevLevel > $currLevel)
				for($i = $prevLevel; $i > $currLevel; $i--)
					$this->renderClose();

			($cnt > 0) ? $this->renderParent($row) : $this->renderChild($row);
			$prevLevel = $currLevel;
		}
		for($i = $prevLevel; $i > 0; $i--)
			$this->renderClose();
		echo '</ul>';
	}

	protected function renderParent($row)
	{
		$c = $this->showCount ? ' ('.$row[$this->countField].')' : '';

		echo '<li class="alxd-active">';
		echo '<div>';
		echo '<a href="#"><span>&nbsp;</span>'.$row[$this->titleField].$c.'</a>';
		echo '<div class="alxd-action-menu">&nbsp;';
		$this->renderActions('parent', $row[$this->idField]);
		echo '</div>';
		echo '</div>';
		echo '<ul>';
	}

	protected function renderChild($row)
	{
		echo '<li>';
		echo '<div>';
		echo '<a>'.$row[$this->titleField].'</a>';
		echo '<div class="alxd-action-menu">&nbsp;';
		$this->renderActions('child', $row[$this->idField]);
		echo '</div>';
		echo '</div>';
		echo '</li>';
	}

	protected function renderClose()
	{
		echo '</ul>';
		echo '</li>';
	}

	private function renderActions($for, $id)
	{
		if (is_array($this->actions))
		{
			echo '<ul>';
			foreach($this->actions as $action)
			{
				$f = array_key_exists('for', $action) ? $action['for'] : '';
				if (strlen($f) == 0 || strcasecmp($f, $for) == 0)
				{
					echo '<li>';
					echo '<a href="'.Yii::app()->controller->createUrl($action['action'], $_GET + array('id'=>$id)).'">'.$action['label'].'</a>';
					echo '</li>';
				}
			}
			echo '</ul>';
		}
	}
}

?>